import logging
from datetime import datetime

import pytz
from telethon import events

from bot_s3rius.config import config
from bot_s3rius.utils.responses import mark_unread

logger = logging.getLogger(__name__)


@config.telegram_client.on(events.NewMessage(pattern=r"^\.h$"))
@mark_unread
async def show_help(event):
    with open(config.project_dir / "static_messages/help.txt") as f:
        contents = f.read()
    await event.respond(contents)


@config.telegram_client.on(events.NewMessage(pattern=r"^\.cid$"))
@mark_unread
async def get_chat_id(event):
    chat = await event.get_chat()
    await event.respond(str(chat.id))


@config.telegram_client.on(events.NewMessage(pattern=r"^\.t$"))
@mark_unread
async def show_time(event):
    tz = pytz.timezone("Europe/Moscow")
    now = datetime.utcnow()
    tz_offset = tz.utcoffset(now)
    current_time = now + tz_offset
    await event.respond(f"{current_time.strftime('%Y-%M-%d %H:%M:%S+03')} MSK")


@config.telegram_client.on(events.NewMessage(pattern=".*@all.*", forwards=False))
@mark_unread
async def mention_all(event: events.NewMessage.Event):
    if event.is_group:
        chat = await event.get_chat()
        participants = []
        async for participant in config.telegram_client.iter_participants(chat):
            if participant.username:
                if participant.username.lower().endswith("bot"):
                    continue
                participants.append(f"@{participant.username}")
            else:
                participants.append(
                    f"[{participant.first_name}](tg://user?id={participant.id})"
                )
        await event.respond(" ".join(participants))
