import logging
import re
import urllib.parse
from decimal import Decimal

import httpx
from telethon import events

from bot_s3rius.config import config
from bot_s3rius.utils.converters import get_info_from_group
from bot_s3rius.utils.responses import mark_unread

logger = logging.getLogger(__name__)

TARGET_CURRENCY = "rub"

CURRENCY_MAPPING = {
    "btc": "bitcoin",
    "биткоин": "bitcoin",
    "биткоина": "bitcoin",
    "биткоинов": "bitcoin",
    "биток": "bitcoin",
    "битка": "bitcoin",
    "битков": "bitcoin",
    "etc": "ethereum",
    "ефир": "ethereum",
    "эфир": "ethereum",
    "эфира": "ethereum",
    "b5k": "bitcoin-5000",
    "bvk": "bitcoin-5000",
    "ltc": "litecoin",
    "лайт": "litecoin",
}

SUPPORTED_CURRENCIES = ["bitcoin", "ethereum", "litecoin", "bitcoin-5000"]

CANONICAL_NAME = {
    "bitcoin": "BTC",
    "ethereum": "ETH",
    "litecoin": "LTC",
    "bitcoin-5000": "BVK",
}

CURRENCY_PATTERN = (
    rf"\s*((\d+)([\.,]\d+)?)\s+("
    rf"{'|'.join(SUPPORTED_CURRENCIES + list(CURRENCY_MAPPING.keys()))})"
)


@config.telegram_client.on(
    events.NewMessage(
        pattern=re.compile(f".*{CURRENCY_PATTERN}.*", re.IGNORECASE | re.DOTALL)
    )
)
@mark_unread
async def replace_crypto_currency(event: events.NewMessage.Event):
    groups = re.findall(CURRENCY_PATTERN, event.message.message, re.IGNORECASE)
    response = "**Полагаясь на текущий курс валют могу сказать следующее:**\n\n"
    currency_mapper = []
    groups_info = list(map(lambda x: get_info_from_group(x, CURRENCY_MAPPING), groups))
    names = ",".join(set([x.name for x in groups_info]))
    query_params = {"ids": names, "vs_currencies": TARGET_CURRENCY}
    async with httpx.AsyncClient() as client:
        query = urllib.parse.urlencode(query_params)
        resp = await client.get(
            f"https://api.coingecko.com/api/v3/simple/price?{query}"
        )
    currency_exchange = resp.json()
    logger.debug(currency_exchange)
    for currency in groups_info:
        val_id = f"{currency.val} {currency.name}"
        logger.debug(val_id)
        new_val = (
            Decimal(currency_exchange[currency.name][TARGET_CURRENCY]) * currency.val
        )
        if val_id not in currency_mapper:
            canon_name = CANONICAL_NAME[currency.name]
            if currency.sci:
                response += (
                    f"`{currency.source_val} {canon_name}"
                    f" = {new_val.to_eng_string()} RUB`\n"
                )
            else:
                response += (
                    f"`{currency.source_val} {canon_name} = {new_val:.2f} RUB`\n"
                )
            currency_mapper.append(val_id)

    await event.reply(response)


@config.telegram_client.on(events.NewMessage(pattern="^.scc$"))
@mark_unread
async def supported_currencies(event):
    async with httpx.AsyncClient() as client:
        response = await client.get("https://api.coingecko.com/api/v3/coins/list")
        raw_json = response.json()
    currencies = {x["id"]: x for x in raw_json}
    response = "**Поддерживаемые курсы криптовалют**:\n\n"
    for curr in SUPPORTED_CURRENCIES:
        response += f"{CANONICAL_NAME[curr]} ({currencies[curr.lower()]['name']})\n"

    await event.reply(response)
