import logging
import re
from decimal import Decimal

import httpx
from telethon import events

from bot_s3rius.config import config
from bot_s3rius.utils.converters import get_info_from_group
from bot_s3rius.utils.responses import mark_unread

logger = logging.getLogger(__name__)

CURRENCY_MAPPING = {
    "фунт": "GBP",
    "фунта": "GBP",
    "фунтов": "GBP",
    "бакс": "USD",
    "бакса": "USD",
    "баксов": "USD",
    "доллар": "USD",
    "доллара": "USD",
    "долларов": "USD",
    "евро": "EUR",
    "иена": "JPY",
    "иены": "JPY",
    "иен": "JPY",
    "йена": "JPY",
    "йены": "JPY",
    "йен": "JPY",
    "вон": "KRW",
    "воны": "KRW",
    "франк": "CHF",
    "франка": "CHF",
    "франков": "CHF",
    "крона": "SEK",
    "кроны": "SEK",
    "крон": "SEK",
    "юань": "CNY",
    "юаня": "CNY",
    "юаней": "CNY",
    "гривна": "UAH",
    "гривны": "UAH",
    "гривен": "UAH",
    "грiвен": "UAH",
}

SUPPORTED_CURRENCIES = [
    "GBP",
    "HUF",
    "USD",
    "EUR",
    "CNY",
    "NOK",
    "UAH",
    "SEK",
    "CHF",
    "KRW",
    "JPY",
]

CURRENCY_PATTERN = (
    rf"\s*((\d+)([\.,]\d+)?)\s+("
    rf"{'|'.join(SUPPORTED_CURRENCIES + list(CURRENCY_MAPPING.keys()))})"
)


@config.telegram_client.on(
    events.NewMessage(
        pattern=re.compile(f".*{CURRENCY_PATTERN}.*", re.IGNORECASE | re.DOTALL)
    )
)
@mark_unread
async def replace_currency(event):
    async with httpx.AsyncClient() as f:
        current_currency = await f.get("https://www.cbr-xml-daily.ru/daily_json.js")
    response_json = current_currency.json()
    currencies = response_json["Valute"]
    groups = re.findall(CURRENCY_PATTERN, event.message.message, re.IGNORECASE)
    response = "**Полагаясь на текущий курс валют могу сказать следующее:**\n\n"
    currency_mapper = []
    for group in groups:
        c_info = get_info_from_group(group, CURRENCY_MAPPING)
        val_id = f"{c_info.val} {c_info.name}"
        new_val = (
            c_info.val
            * Decimal(currencies[c_info.name]["Value"])
            / Decimal(currencies[c_info.name]["Nominal"])
        )
        if val_id not in currency_mapper:
            if c_info.sci:
                response += (
                    f"`{c_info.val} {c_info.name} = {new_val.to_eng_string()} RUB`\n"
                )
            else:
                response += f"`{c_info.val} {c_info.name} = {new_val:.2f} RUB`\n"
            currency_mapper.append(val_id)
    await event.reply(response)


@config.telegram_client.on(events.NewMessage(pattern="^.sc$"))
@mark_unread
async def supported_currencies(event):
    async with httpx.AsyncClient() as f:
        current_currency = await f.get("https://www.cbr-xml-daily.ru/daily_json.js")
    response_json = current_currency.json()
    currencies = response_json["Valute"]
    response = "**Поддерживаемые курсы валют**:\n\n"
    for curr in SUPPORTED_CURRENCIES:
        response += f"{curr} ({currencies[curr]['Name']})\n"

    await event.reply(response)
