import logging
from urllib.parse import urlencode

import httpx
from telethon import events

from bot_s3rius.config import config
from bot_s3rius.utils.responses import failed_search_answer, mark_unread, thinking

logger = logging.getLogger(__name__)


@config.telegram_client.on(events.NewMessage(pattern=r"^\.i (.*)", forwards=False))
@mark_unread
@thinking
async def search_images(event: events.NewMessage.Event):  # noqa: C901
    query_str = event.pattern_match.group(1).strip()
    params = {
        "count": "30",
        "q": query_str,
        "t": "images",
        "locale": "ru_RU",
        "uiv": "5",
    }
    async with httpx.AsyncClient() as client:
        search_res = await client.get(
            f"https://api.qwant.com/api/search/images?{urlencode(params)}",
            headers={"User-Agent": "S3rius bot 1.0"},
        )
    if search_res.status_code == 429:
        logger.warning("Limit is reached")
        await event.reply(
            "Кажись они прознали что я бот. Кто-нибудь решите капчу за меня."
        )
    if search_res.status_code != 200:
        logger.warning("Image search status code is not 200")
        logger.debug(search_res.content)
        await event.reply(failed_search_answer())
        return
    response_json = search_res.json()
    if response_json["status"] != "success":
        logger.warning("Search status is not 'success'")
        await event.reply(failed_search_answer())
        return
    resp_data = response_json["data"]
    if resp_data["result"]["total"] < 1:
        await event.reply(failed_search_answer())
        return
    images_resp = resp_data["result"]["items"]
    images = []
    async with httpx.AsyncClient() as client:
        for item in images_resp[:10]:
            if "media" in item:
                logger.debug(item["media"])
                try:
                    test_img_res = await client.get(item["media"])
                    if test_img_res.status_code == 200:
                        img_data = test_img_res.content
                        images.append(img_data)
                except Exception as e:
                    logger.warning(e)
    chat = await event.get_chat()
    await event.client.send_file(chat, images)
