from pathlib import Path
from typing import List

from pydantic import BaseSettings
from pydantic.fields import Field
from telethon import TelegramClient

__all__ = ["config"]


class Config(BaseSettings):
    telegram_app_id: str = Field(None, env="BOT_TELEGRAM_APP_ID", required=True)
    telegram_api_hash: str = Field(None, env="BOT_TELEGRAM_API_HASH", required=True)
    libraries_io_token: str = Field(
        None, env="BOT_LIBRARIES_IO_API_TOKEN", required=True
    )
    telegram_account_phone: str = Field(
        None, env="BOT_TELEGRAM_ACCOUNT_PHONE", required=True
    )
    app_mode: str = Field(name="app_mode", default="dev", env="BOT_APP_MODE")
    debug_mode: bool = False
    flask_port: str = Field(name="flask_port", default="8080", env="BOT_FLASK_PORT")
    excluded_chats: List[str] = Field([], env="BOT_EXCLUDED_CHATS")
    telegram_client: TelegramClient = None

    @property
    def project_dir(self) -> Path:
        return Path(__file__).parent


config = Config()
