import logging
import random
import uuid

from flask import Flask, current_app, render_template, request, send_file

from bot_s3rius.config import config

logger = logging.getLogger(__name__)


def form_template():
    context = {"activated": current_app.activated, "cash_dropper": str(uuid.uuid4())}
    return render_template("index.html", **context)


def girl_image():
    num = random.randint(1, 3)
    return send_file(
        config.project_dir / f"templates/static/girl_{num}.png",
        mimetype="image/png",
    )


def get_code():
    code = request.form.get("code", None)
    if code is not None:
        logger.debug("Updating token!")
        current_app.update_token(code)
    return "Ok"


class MyServer(Flask):
    def __init__(self, token_value, *args, **kwargs):
        super(MyServer, self).__init__(*args, **kwargs)
        self.token_value = token_value
        self.activated = False
        self.init_routes()

    def init_routes(self):
        self.add_url_rule("/", None, form_template)
        self.add_url_rule("/girl.png", None, girl_image)
        self.add_url_rule("/code", None, get_code, methods=["POST"])

    def update_token(self, new_val):
        self.token_value.value = new_val
        self.activated = True


def create_web_app(token):
    app = MyServer(token, __name__)
    app.run("0.0.0.0", config.flask_port)
