import asyncio
import functools
import logging
import random

from telethon import events, functions

from bot_s3rius.config import config

logger = logging.getLogger(__name__)

FAILED_SEARCH = [
    "Что-то я ничего не нашел.",
    "Сколько не искал, так ничего и на нашел.",
    "Не ищется такая тема.",
    "Странный запрос. У меня ничего не нашло.",
    "Перепроверь запрос, у меня в поске пусто.",
]

THINKING_MSGS = [
    "Кхммм...",
    "Сейчас-сейчас",
    "Уже почти вижу результат.",
    "А это законно, кстати?",
    "Что-то лезет по моему TCP.",
    "Какой большой пейлоад. Прям не знаю, смогу ли я...",
    "Пока я ищу отправьте Леночке привет.",
    "Шота какта сложна думаеца.",
    "Ждем-с...",
]


def failed_search_answer() -> str:
    return random.choice(FAILED_SEARCH)


def thinking(f):  # noqa: C901
    @functools.wraps(f)
    async def updated_function(event: events.NewMessage.Event):
        think_msg = random.choice(THINKING_MSGS)
        message = await event.reply(think_msg)

        async def update_thinking_msg():
            while True:
                await asyncio.sleep(3)
                valid_choices = set(THINKING_MSGS) - {think_msg}
                await message.edit(random.choice(list(valid_choices)))

        thinking_task = asyncio.create_task(update_thinking_msg())
        main_task = asyncio.create_task(f(event))
        task_iterator = asyncio.as_completed([main_task, thinking_task])
        try:
            await next(task_iterator)
        except Exception as e:
            logger.warning(f"Found error while thinking: {e}")
        thinking_task.cancel()
        try:
            await thinking_task
        except asyncio.CancelledError:
            logger.debug("Thinking cancelled.")
        await message.delete()

    return updated_function


def mark_unread(f):
    @functools.wraps(f)
    async def wrapper(event: events.NewMessage.Event):
        try:
            await f(event)
        except Exception as e:
            logger.error(f"Exception found: {e}")
        chat = await event.get_chat()
        functions.messages.MarkDialogUnreadRequest(chat, unread=True)

    return wrapper


def danger_message(f):
    @functools.wraps(f)
    async def wrapper(event: events.NewMessage.Event):
        chat = await event.get_chat()
        if str(chat.id) in set(config.excluded_chats):
            logger.debug("Skipping sending message")
            return
        await f(event)

    return wrapper


def check_if_only_smiles(message: str):
    return all([x in ")0." for x in message])
